diff -Nru svn/src/callbacks.c openmoko-messages/src/callbacks.c
--- svn/src/callbacks.c	2007-04-11 13:45:12.000000000 +0800
+++ openmoko-messages/src/callbacks.c	2007-04-11 11:31:57.000000000 +0800
@@ -87,7 +87,7 @@
   g_signal_connect (G_OBJECT(sms_window->addressBtn),
                     "clicked",
                     G_CALLBACK(cb_addressBtn_clicked),
-                    sms_window->toEntry );
+                    sms_window);
 }
 
 void cb_new_mail (GtkMenuItem* item, MessengerData* d)
@@ -100,7 +100,7 @@
   g_signal_connect (G_OBJECT(mail_window->addressBtn),
                     "clicked",
                     G_CALLBACK(cb_addressBtn_clicked),
-                    mail_window->toEntry);
+                    mail_window);
 }
 
 void cb_new_folder (GtkMenuItem* item, MessengerData* d)
@@ -588,9 +588,15 @@
 
 void cb_addressBtn_clicked (GtkButton* button, gpointer* data)
 {
-  GtkWidget* contactWin = sms_contact_new();
+  SmsDialogWindow* dialogWin = (SmsDialogWindow*)data;
+  gchar* title = sms_dialog_window_get_title(dialogWin);
+  GtkWidget* contactWin;
+  if(!g_strcasecmp(title,"New SMS"))
+    contactWin = sms_contact_new(CONTACT_SMS);
+  else if(!g_strcasecmp(title,"New Email"))
+    contactWin = sms_contact_new(CONTACT_EMAIL);
   gtk_window_set_decorated (GTK_WINDOW(contactWin),FALSE);
-  GtkWidget* toEntry = (GtkWidget*)data;
+  GtkWidget* toEntry = dialogWin->toEntry;
   g_signal_connect (G_OBJECT(contactWin),"contact_select_done",
                     G_CALLBACK(sms_contact_select_done), toEntry);
   gtk_widget_show(contactWin);
diff -Nru svn/src/sms-contact-window.c openmoko-messages/src/sms-contact-window.c
--- svn/src/sms-contact-window.c	2007-04-11 13:45:12.000000000 +0800
+++ openmoko-messages/src/sms-contact-window.c	2007-04-11 11:31:57.000000000 +0800
@@ -23,6 +23,11 @@
 
 #define SMS_CONTACT_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SMS_TYPE_CONTACT_WINDOW, SmsContactWindowPrivate))
 
+enum{
+  PROP_0,
+  PROP_TYPE
+};
+
 typedef struct
 {
   /* private members define here */
@@ -44,6 +49,15 @@
 static void contact_select_done(SmsContactWindow* self);
 gboolean get_selected_contact (GtkTreeModel* model, GtkTreePath* path, 
                                GtkTreeIter* iter, gpointer data);
+static void open_book_syn (SmsContactData* data);
+static void sms_contact_window_set_property (GObject*      object,
+                                             guint         prop_id,
+                                             const GValue* value,
+                                             GParamSpec*   pspec);
+static void sms_contact_window_get_property (GObject*    object,
+                                             guint       prop_id,
+                                             GValue*     value,
+                                             GParamSpec* pspec);
 
 static void
 sms_contact_window_dispose(GObject* object)
@@ -58,6 +72,51 @@
   G_OBJECT_CLASS (sms_contact_window_parent_class)->finalize (object);
 }
 
+static void sms_contact_window_set_property (GObject*      object,
+                                             guint         prop_id,
+                                             const GValue* value,
+                                             GParamSpec*   pspec)
+{
+  SmsContactWindow* window = SMS_CONTACT_WINDOW(object);
+  SmsContactWindowPrivate* priv = SMS_CONTACT_WINDOW_GET_PRIVATE(window);
+
+  switch (prop_id)
+  {
+      case PROP_TYPE:
+          (*priv->data).type = g_value_get_int(value);
+          GtkTreeView* treeView = moko_navigation_list_get_tree_view(priv->contacts_view);
+          GtkTreeViewColumn* column = gtk_tree_view_get_column (treeView, 1);
+          if ((*priv->data).type == CONTACT_SMS)
+            gtk_tree_view_column_set_title (column, "Cell Phone");
+          else
+            gtk_tree_view_column_set_title (column, "Email Address");
+          gtk_widget_show_all (treeView);
+          open_book_syn(priv->data);
+          break;
+      default:
+          G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+          break;
+  }
+}
+
+static void sms_contact_window_get_property (GObject*    object,
+                                             guint       prop_id,
+                                             GValue*     value,
+                                             GParamSpec* pspec)
+{
+  SmsContactWindow* window = SMS_CONTACT_WINDOW(object);
+  SmsContactWindowPrivate* priv = SMS_CONTACT_WINDOW_GET_PRIVATE(window);
+
+  switch (prop_id)
+  {
+      case PROP_TYPE:
+          g_value_set_int (value, (*priv->data).type);
+          break;
+      default:                                                            
+          G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+          break;
+  }
+}
 
 static void
 sms_contact_window_class_init (SmsContactWindowClass* klass)
@@ -77,12 +136,21 @@
 
   object_class->dispose = sms_contact_window_dispose;
   object_class->finalize = sms_contact_window_finalize;
+  object_class->set_property = sms_contact_window_set_property;
+  object_class->get_property = sms_contact_window_get_property;
 
+  g_object_class_install_property (object_class,
+                                   PROP_TYPE,
+                                   g_param_spec_int("type",
+                                                    "type",
+                                                    "contact window type",
+                                                    0,1,1,
+                                                    G_PARAM_READWRITE));
 }
 
-GtkWidget* sms_contact_new()
+GtkWidget* sms_contact_new(ContactType type)
 {
-  return GTK_WIDGET(g_object_new(SMS_TYPE_CONTACT_WINDOW, NULL));
+  return GTK_WIDGET(g_object_new(SMS_TYPE_CONTACT_WINDOW, "type", type, NULL));
 }
 
 static void updateContactsView (EBook* book, EBookStatus status, 
@@ -92,22 +160,44 @@
   SmsContactData* data = (SmsContactData*)closure;
   GtkListStore *contacts_liststore = data->contacts_liststore;
   GList* c = contacts;
-  data->contacts = contacts;
+  data->contacts = g_list_copy(contacts);
+  ContactType type = (*data).type;
   g_debug ("list length %d", g_list_length(c));
+  g_debug ("window type %d", (*data).type);
 
-  const gchar *uid, *name , *phoneNum;
+  const gchar *uid, *name, *phoneNum, *email;
   for (;c;c=c->next){
     EContact *contact = E_CONTACT (c->data);
     uid = e_contact_get_const (contact, E_CONTACT_UID);
     name = e_contact_get_const (contact, E_CONTACT_FULL_NAME);
     phoneNum =  e_contact_get_const (contact, E_CONTACT_PHONE_MOBILE);
-    gtk_list_store_append(contacts_liststore, &iter);
-    gtk_list_store_set (contacts_liststore, &iter, 
-                        CONTACT_SEL_COL, FALSE,
-                        CONTACT_UID_COL, uid, 
-                        CONTACT_NAME_COL, name,
-			CONTACT_CELLPHONE_COL, phoneNum,
-			-1);
+    email = e_contact_get_const (contact, E_CONTACT_EMAIL_1);
+    /*look up window type
+     * if type is sms and phoneNum isn't empty
+     *     add to list view
+     * if type is email and email isn't empty
+     *     add to list view
+     * other condition just add to list view*/
+    if ((type == CONTACT_SMS) && (phoneNum != NULL)){
+      gtk_list_store_append(contacts_liststore, &iter);
+      gtk_list_store_set (contacts_liststore, &iter,
+                          CONTACT_SEL_COL, FALSE,
+                          CONTACT_UID_COL, uid,
+                          CONTACT_NAME_COL, name,
+                          CONTACT_CELLPHONE_COL, phoneNum,
+                          -1);
+    }else if ((type == CONTACT_EMAIL) && (email != NULL)){
+      gtk_list_store_append(contacts_liststore, &iter);
+      gtk_list_store_set (contacts_liststore, &iter,
+                          CONTACT_SEL_COL, FALSE,
+                          CONTACT_UID_COL, uid,
+                          CONTACT_NAME_COL, name,
+                          CONTACT_CELLPHONE_COL, email,
+                          -1);
+    }else {
+      data->contacts=g_list_remove(data->contacts, contact);
+      continue;
+    }
   }
 }
 
@@ -167,10 +257,10 @@
   /* initialzation */
   SmsContactWindowPrivate* priv = SMS_CONTACT_WINDOW_GET_PRIVATE(self);
   priv->vbox = gtk_vbox_new(FALSE,0);
-  SmsContactData* data = g_new0(SmsContactData,1);
-  priv->data = data;
-  data->book = e_book_new_system_addressbook(NULL);
-  if (!data->book)
+  //SmsContactData* data = g_new0(SmsContactData,1);
+  priv->data = g_new0(SmsContactData,1);
+  priv->data->book = e_book_new_system_addressbook(NULL);
+  if (!priv->data->book)
     g_critical ("Could not load system addressbook");
 
   /* Set the "select" window title */
@@ -182,11 +272,11 @@
   gtk_box_pack_start(GTK_BOX(priv->vbox), eventbox, FALSE, TRUE, 0);
 
   /* create contact list */
-  data->contacts_liststore = gtk_list_store_new (4, G_TYPE_BOOLEAN,
+  priv->data->contacts_liststore = gtk_list_store_new (4, G_TYPE_BOOLEAN,
                                                  G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-  priv->contacts_view = create_contacts_list(data);
+  priv->contacts_view = create_contacts_list(priv->data);
   gtk_box_pack_start(GTK_BOX(priv->vbox), priv->contacts_view, TRUE, TRUE, 0);
-  g_object_unref (data->contacts_liststore);
+  g_object_unref (priv->data->contacts_liststore);
 
   /* add close and back window button */
   GtkWidget* closebutton = moko_pixmap_button_new();
@@ -202,7 +292,7 @@
   
   /* loading contacts */
   //g_idle_add((GSourceFunc)open_book,data);
-  open_book_syn (data);
+  //open_book_syn (data);
 
   gtk_widget_show_all(priv->vbox);
   gtk_container_add(GTK_CONTAINER(self), priv->vbox);
diff -Nru svn/src/sms-contact-window.h openmoko-messages/src/sms-contact-window.h
--- svn/src/sms-contact-window.h	2007-04-11 13:45:12.000000000 +0800
+++ openmoko-messages/src/sms-contact-window.h	2007-04-11 11:31:57.000000000 +0800
@@ -50,6 +50,14 @@
   LAST_SIGNAL
 };
 
+enum {
+  CONTACT_SMS,
+  CONTACT_EMAIL,
+  CONTACT_LAST_TYPE
+};
+
+typedef gint ContactType;
+
 typedef struct
 {
   GtkWindow parent;
@@ -68,10 +76,11 @@
   EBook* book;
   GtkListStore *contacts_liststore;
   GList* contacts;
+  ContactType type;
 }SmsContactData;
 
 GType sms_contact_window_get_type();
-GtkWidget* sms_contact_new();
+GtkWidget* sms_contact_new(ContactType type);
 GList* sms_get_selected_contacts(SmsContactWindow* self);
 
 G_END_DECLS
diff -Nru svn/src/sms-dialog-window.c openmoko-messages/src/sms-dialog-window.c
--- svn/src/sms-dialog-window.c	2007-04-11 13:45:12.000000000 +0800
+++ openmoko-messages/src/sms-dialog-window.c	2007-04-11 11:31:57.000000000 +0800
@@ -394,4 +394,10 @@
   return FALSE;
 }
 
+gchar* sms_dialog_window_get_title(SmsDialogWindow* self)
+{
+  SmsDialogWindowPrivate* priv = SMS_DIALOG_WINDOW_GET_PRIVATE(self);
+  gchar* title = g_strdup(gtk_label_get_text(GTK_LABEL(priv->titleLabel)));
+  return title; 
+}
 
diff -Nru svn/src/sms-dialog-window.h openmoko-messages/src/sms-dialog-window.h
--- svn/src/sms-dialog-window.h	2007-04-11 13:45:12.000000000 +0800
+++ openmoko-messages/src/sms-dialog-window.h	2007-04-11 11:31:57.000000000 +0800
@@ -57,6 +57,7 @@
 void sms_dialog_window_set_contents(SmsDialogWindow* self, GtkWidget* contents);
 void sms_dialog_reply_message(SmsDialogWindow* self, message* msg);
 void sms_dialog_forward_message(SmsDialogWindow* self, message* msg);
+gchar* sms_dialog_window_get_title(SmsDialogWindow* self);
 
 G_END_DECLS
 
